use Physics::Unit::Scalar;

$d = new Physics::Unit::Distance('98000000 mi');
$s = new Physics::Unit::Speed('45mi/fortnight');
$m = new Physics::Unit::Mass('205lbm');
$a = new Physics::Unit::Acceleration('205 furlongs/fortnight^2');
$i = new Physics::Unit::Time('205 score');

